*DECK SCRNDF
      SUBROUTINE SCRNDF(IMPX,NBISO1,ISO,IBM,INOMIS,IPSAP,IPLIB,NCAL,
     1 TERP,MY1,MY2,YLDS,ISTYP)
*
*-----------------------------------------------------------------------
*
*Purpose:
* store records PIFI and PYIELD into a Microlib.
*
*Copyright:
* Copyright (C) 2015 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IMPX    print parameter (equal to zero for no print).
* NBISO1  number of particularized isotopes.
* ISO     particularized isotope index.
* IBM     material mixture.
* INOMIS  array containing the names of the particularized isotopes.
* IPSAP   address of the Saphyb object.
* IPLIB   address of the output microlib LCM object.
* NCAL    number of elementary calculations in the Saphyb.
* TERP    interpolation factors.
* MY1     number of fissile isotopes including macroscopic sets.
* MY2     number of fission fragment.
* YLDS    fission yields.
*
*Parameters: output
* ISTYP   type of isotope ISO (=2: fissile; =3: fission product).
*
*-----------------------------------------------------------------------
*
      USE GANLIB
*----
*  SUBROUTINE ARGUMENTS
*----
      TYPE(C_PTR) IPSAP,IPLIB
      INTEGER IMPX,NBISO1,ISO,IBM,INOMIS(2,NBISO1),NCAL,MY1,MY2,ISTYP
      REAL TERP(NCAL)
      DOUBLE PRECISION YLDS(MY1,MY2)
*----
*  LOCAL VARIABLES
*----
      CHARACTER HNISO*12,TEXT12*12
*----
*  ALLOCATABLE AYYAYS
*----
      INTEGER, ALLOCATABLE, DIMENSION(:) :: ADRY
      INTEGER, ALLOCATABLE, DIMENSION(:,:) :: IPYNAM
      REAL, ALLOCATABLE, DIMENSION(:) :: PYIELD
*
      WRITE(HNISO,'(2A4,I4.4)') (INOMIS(I0,ISO),I0=1,2),IBM
      ISTYP=0
      DO 10 ICAL=NCAL,1,-1
      IF(TERP(ICAL).EQ.0.0) GO TO 10
      WRITE(TEXT12,'(4Hcalc,I8)') ICAL
      CALL LCMSIX(IPSAP,TEXT12,1) ! step up to calc
      CALL LCMSIX(IPSAP,'info',1)
      CALL LCMGET(IPSAP,'NISY',NISY)
      IF(ISO.GT.NISY) CALL XABORT('SCRNDF: NISY OVERFLOW.')
      ALLOCATE(ADRY(NISY))
      CALL LCMGET(IPSAP,'ADRY',ADRY)
      CALL LCMSIX(IPSAP,' ',2)
      CALL LCMSIX(IPSAP,' ',2)
      IF(ADRY(ISO).GT.0) THEN
*       ISO is a fissile isotope
        ISTYP=2
      ELSE IF(ADRY(ISO).LT.0) THEN
*       ISO is a fission product
        ISTYP=3
        IY2=-ADRY(ISO)
        IF(IY2.GT.MY2) CALL XABORT('SCRNDF: MY2 OVERFLOW.')
        ALLOCATE(IPYNAM(3,MY1),PYIELD(MY1))
        CALL XDISET(IPYNAM,3*MY1,0)
        CALL XDRSET(PYIELD,MY1,0)
        IF(IMPX.GT.2) THEN
          WRITE(6,'(25H SCRNDF: fission product=,A12)') HNISO
        ENDIF
        DO JSO=1,NISY
          IF(ADRY(JSO).GT.0) THEN
            IY1=ADRY(JSO)
            IF(IY1.GT.MY1) CALL XABORT('SCRNDF: MY1 OVERFLOW.')
            WRITE(TEXT12,'(2A4,I4.4)') (INOMIS(I0,JSO),I0=1,2),IBM
            READ(TEXT12,'(3A4)') (IPYNAM(I0,IY1),I0=1,3)
            PYIELD(IY1)=REAL(YLDS(IY1,IY2))
            IF(IMPX.GT.2) THEN
              WRITE(6,'(9X,16Hfissile isotope(,I4,2H)=,A12)') IY1,TEXT12
            ENDIF
          ENDIF
        ENDDO
        CALL LCMPUT(IPLIB,'PYNAM',3*MY1,3,IPYNAM)
        CALL LCMPUT(IPLIB,'PYIELD',MY1,2,PYIELD)
        IF(IMPX.GT.2) THEN
          WRITE(6,'(3X,7HPYIELD=,1P,8E12.4/(8X,10E12.4))') (PYIELD(I),
     1    I=1,MY1)
        ENDIF
        DEALLOCATE(PYIELD,IPYNAM)
      ENDIF
      DEALLOCATE(ADRY)
      RETURN
   10 CONTINUE
      CALL XABORT('SCRNDF: UNABLE TO FIND A CALCULATION DIRECTORY.')
      RETURN
      END
