!!!!!
!
! fichier g2s_generatingTrack.md9
!
! Ce module sert a generer les donnees relatives aux gigognes de provenance des
! nodes, et aux numeros type dragon qu'elles auraient pris
!
! fonctions:
!  - generateTrack : fonction d'entree du module
!  - calculFinalMerge : calcul de la numerotation type dragon
!  - ltVec : fonction d'odre specifique pour des vecteurs d'entiers de tailles
!            differentes
!
!!!!!

module track

    use cellulePlaced
    use segArc
    use ptNodes
    implicit none

contains
  subroutine generateTrack(szP,szSa,nbNode,lgMaxGig,gig,merg)
    integer,intent(in) :: szP,szSa,nbNode,lgMaxGig
    integer,dimension(lgMaxGig*nbNode),intent(out) :: gig
    integer,dimension(nbNode),intent(out) :: merg
    
    integer :: i,lgMaxMrg,s,d,lgMaxGigTest
    integer,dimension(:,:),allocatable :: mrgMat
    
    lgMaxGigTest = 0
    lgMaxMrg = 0
    do i = 1,szP
       lgMaxGigTest = max(lgMaxGigTest,size(tabCellulePlaced(i)%p%gig))
       lgMaxMrg = max(lgMaxMrg,size(tabCellulePlaced(i)%p%mrg))
    end do
    if(lgMaxGigTest /= lgMaxGig) call XABORT('g2s_generatingTrack: lgMax error')
    lgMaxMrg = lgMaxMrg + 1
    allocate(mrgMat(lgMaxMrg,nbNode))
    gig = 0
    mrgMat = 0
    do i = 1,szSa
       if (tabSegArc(i)%p%nodeg>0.and.tabSegArc(i)%p%indCellPg>0) then
          s = size(tabCellulePlaced(tabSegArc(i)%p%indCellPg)%p%gig)
          d = (tabSegArc(i)%p%nodeg-1)*lgMaxGig+1
          gig(d:d+s-1) = tabCellulePlaced(tabSegArc(i)%p%indCellPg)%p%gig(1:s)
          s = size(tabCellulePlaced(tabSegArc(i)%p%indCellPg)%p%mrg)
          d = tabSegArc(i)%p%nodeg
          mrgMat(1:s,d) = tabCellulePlaced(tabSegArc(i)%p%indCellPg)%p%mrg(1:s)
          mrgMat(s+1,d) = tabNodeGigSect(d)%merge
       end if
       if (tabSegArc(i)%p%noded>0.and.tabSegArc(i)%p%indCellPd>0) then
          s = size(tabCellulePlaced(tabSegArc(i)%p%indCellPd)%p%gig)
          d = (tabSegArc(i)%p%noded-1)*lgMaxGig+1
          gig(d:d+s-1) = tabCellulePlaced(tabSegArc(i)%p%indCellPd)%p%gig(1:s)
          s = size(tabCellulePlaced(tabSegArc(i)%p%indCellPd)%p%mrg)
          d = tabSegArc(i)%p%noded
          mrgMat(1:s,d) = tabCellulePlaced(tabSegArc(i)%p%indCellPd)%p%mrg(1:s)
          mrgMat(s+1,d) = tabNodeGigSect(d)%merge
       end if
    end do
    call calculFinalMerge(mrgMat,merg)
    deallocate(mrgMat,tabNodeGigSect)
  end subroutine generateTrack

  subroutine calculFinalMerge(inMat,outVec)
    implicit none
    integer,dimension(:,:),intent(in) :: inMat
    integer,dimension(:),intent(out)  :: outVec
    
    integer :: i,j,d1,d2,maxD2
    logical :: found,sorted
    integer,dimension(:),allocatable   :: tmpVec
    integer,dimension(:,:),allocatable :: workMat
    
    d1 = size(inMat,1) !profondeur max de gigogne + 1
    d2 = size(inMat,2) !nombre de nodes
    maxD2 = 0
    allocate(workMat(d1,d2),tmpVec(d1))
    workMat = 0
    !remplissage de workMat a l'aide d'occurences uniques de lignes de inMat
    do i = 1,d2
       found = .false.
       do j = 1,maxD2
          if (all(workMat(:,j)==inMat(:,i))) then
             found = .true.
             exit
          end if
       end do
       if (.not. found) then
          maxD2 = maxD2 + 1
          workMat(:,maxD2) = inMat(:,i)
       end if
    end do
    !classement des lignes de workMat en ordre croissant par bubble-sort
    do j = maxD2,2,-1
       sorted = .true.
       do i = 1,j-1
          if (ltVec(workMat(:,i+1),workMat(:,i))) then
             tmpVec(:)      = workMat(:,i+1)
             workMat(:,i+1) = workMat(:,i)
             workMat(:,i)   = tmpVec(:)
             sorted = .false.
          end if
       end do
       if (sorted) exit
    end do
    !remplissage de outVec en fonction de l'egalite entre les lignes de workMat
    !et inMat, apres le classement
    do i = 1,d2
       do j = 1,maxD2
          if (all(workMat(:,j)==inMat(:,i))) then
             outVec(i) = j
             exit
          end if
       end do
    end do
    
    deallocate(workMat,tmpVec)
  end subroutine calculFinalMerge
  
  function ltVec(v1,v2)
    implicit none
    integer,dimension(:),intent(in) :: v1,v2
    logical :: ltVec
    integer :: i
    
    do i = 1,size(v1)
       if (v1(i) < v2(i)) then
          ltVec = .true.
          return
       else if (v1(i) > v2(i)) then
          ltVec = .false.
          return
       end if
    end do
    ltVec = .false.
  end function ltVec
end module track
